class FloodCli
  def initialize(endpoint: nil, proxy: nil, config: nil, verbose: nil)
    @endpoint = endpoint
    @verbose = verbose
    load_configuration(config: config)
    configure_rest_client(proxy: proxy)
  end

  def login(email:, password:)
    response = post '/api/auth',
      {
        email: email,
        password: password
      }

    if response && response.code == 200
      logger.info 'Logged in to Flood IO.'
      config = {
        endpoint: endpoint,
        proxy: @proxy,
        token: JSON.parse(response)['token']
      }
      File.open(@config_file, 'w') { |file| file.write(config.to_yaml) }
    end
  end

  def floods_index
    get 'floods'
  end

  private

  def load_configuration(config:)
    config ||= "#{Dir.home}/.fio"
    if File.file?(config)
      @config_file = config
      @config = YAML::load_file(config)
    end
  end

  def configure_rest_client(proxy:)
    @proxy = proxy if proxy
    RestClient.proxy = @proxy if @proxy
    RestClient.log = STDOUT if @verbose
  end

  def get(url)
    response = RestClient.get [endpoint, url].join('/'), headers_with_token
    parse(response)
    response
    rescue => e
      logger.fatal error(e.response)
      response
  end

  def post(url, params={})
    response = RestClient.post [endpoint, url].join('/'), params, headers
    parse(response)
    response
    rescue => e
      logger.fatal error(e.response)
      response
  end

  def error(response)
    "HTTP #{response.code.to_s} #{JSON.parse(response)['error']}"
    rescue JSON::ParserError
      "HTTP #{response.code.to_s} #{response}"
  end

  def parse(response)
    if response.code == 200
      if @verbose
        logger.ap JSON.parse(response)
      else
        puts JSON.pretty_unparse(JSON.parse(response))
      end
    end
  end

  def token
    raw_token = @config && @config[:token]
    if version == 1
      'Basic ' + Base64.encode64("#{raw_token}:").chomp
    else
      raw_token
    end
  end

  def endpoint
    @endpoint ||= (@config && @config[:endpoint]) || 'https://api.flood.io'
    @endpoint
  end

  def version
    (@config && @config[:version].to_i) || 2
  end

  def headers
    if version == 1
      {
        content_type: :json
      }
    else
      {
        content_type: :json,
        accept: 'application/vnd.flood.v2'
      }
    end
  end

  def headers_with_token
    headers.merge(authorization: token)
  end

  def logger
    @log ||= Logger.new(STDOUT)
    @log.level = @verbose ? Logger::DEBUG : Logger::INFO
    @log.formatter = proc do |severity, datetime, progname, msg|
      "#{datetime}: #{msg}\n"
    end
    @log
  end
end
